@@ -4,7 +4,7 @@
 # Pod stripped from pm file by OODoc 1.05.
 package XML::Compile::Translate::Reader;
 use vars '$VERSION';
-$VERSION = '0.98';
+$VERSION = '0.99';
 
 use base 'XML::Compile::Translate';
 
@@ -312,10 +312,10 @@ sub makeBlockHandler
     # not be an additional nesting in the output tree.
     if($max ne 'unbounded' && $max==1)
     {
-       return ($label => $process) if $min==1;
+        return ($label => $process) if $min==1;
 
         my $code =
-        sub { my $tree = shift or return ();
+        sub { my $tree    = shift or return ();
               my $starter = $tree->currentChild or return ();
               my @pairs   = try { $process->($tree) };
               if($@->wasFatal(class => 'misfit'))
@@ -597,8 +597,9 @@ sub makeComplexElement
     my @elems = odd_elements @$elems;
     my @attrs = (odd_elements(@$attrs), @$attrs_any);
 
-    sub { my $tree    = shift or return ();
-          my $node    = $tree->node;
+    @elems > 1 || @attrs and return
+    sub { my $tree = shift or return ();
+          my $node = $tree->node;
           my %complex
            = ( (map {$_->($tree)} @elems)
              , (map {$_->($node)} @attrs)
@@ -611,6 +612,26 @@ sub makeComplexElement
 
           ($tag => \%complex);
         };
+
+    @elems || return
+    sub { my $tree = shift or return ();
+          defined $tree->currentChild
+              and error __x"element `{name}' not processed at {path}"
+                      , name => $tree->currentType, path => $path
+                      , _class => 'misfit';
+          ($tag => {});
+        };
+
+    my $el = shift @elems;
+    sub { my $tree    = shift or return ();
+          my %complex = $el->($tree);
+          defined $tree->currentChild
+              and error __x"element `{name}' not processed at {path}"
+                      , name => $tree->currentType, path => $path
+                      , _class => 'misfit';
+          ($tag => \%complex);
+        };
+    
 }
 
 #
@@ -1067,15 +1088,16 @@ sub _decodeAfter($$)
 {   my ($self, $path, $call) = @_;
     return $call if ref $call eq 'CODE';
 
-      $call eq 'PRINT_PATH' ? sub {print "$_[2]\n"; $_[1] }
-    : $call eq 'XML_NODE'  ?
-      sub { my $h = $_[1];
+      $call eq 'PRINT_PATH'
+    ? sub {print "$_[2]\n"; $_[1] }
+    : $call eq 'XML_NODE'
+    ? sub { my $h = $_[1];
             $h = { _ => $h } if ref $h ne 'HASH';
             $h->{_XML_NODE} = $_[0];
             $h;
           }
-    : $call eq 'ELEMENT_ORDER' ?
-      sub { my ($xml, $h) = @_;
+    : $call eq 'ELEMENT_ORDER'
+    ? sub { my ($xml, $h) = @_;
             $h = { _ => $h } if ref $h ne 'HASH';
             my @order = map {type_of_node $_}
                 grep { $_->isa('XML::LibXML::Element') }
@@ -1083,13 +1105,19 @@ sub _decodeAfter($$)
             $h->{_ELEMENT_ORDER} = \@order;
             $h;
           }
-    : $call eq 'ATTRIBUTE_ORDER' ?
-      sub { my ($xml, $h) = @_;
+    : $call eq 'ATTRIBUTE_ORDER'
+    ? sub { my ($xml, $h) = @_;
             $h = { _ => $h } if ref $h ne 'HASH';
             my @order = map {$_->nodeName} $xml->attributes;
             $h->{_ATTRIBUTE_ORDER} = \@order;
             $h;
           }
+    : $call eq 'NODE_TYPE'
+    ? sub { my ($xml, $h) = @_;
+            $h = { _ => $h } if ref $h ne 'HASH';
+            $h->{_NODE_TYPE} = type_of_node $xml;
+            $h;
+          }
     : error __x"labeled after hook `{call}' undefined for READER", call=> $call;
 }
 
